#include "main.h"

/******************************************************/
/*                                                    */
/*Procedure de filtrage des dimensions ou celles ayant*/ 
/*des variances sont trop petites sont eliminees      */
/*                                                    */
/******************************************************/

void filtreDimensions(int nbIndividus,int nbDimensionsOrigine,int *nbDimensionsUtilisees,
		      individu_t **individus) {
  int i,j;
  double temp,moyenne,variance,varianceLimite=0.00000001;

  *nbDimensionsUtilisees=0;
  for(i=0;i<nbDimensionsOrigine;i++) {
    moyenne=0;
    for(j=0;j<nbIndividus;j++) {
      moyenne+=individus[j]->valeurs[i];
    }
    moyenne/=(double)nbIndividus;

    variance=0;
    for(j=0;j<nbIndividus;j++) {
      temp=individus[j]->valeurs[i]-moyenne;
      variance+=temp*temp;
    }
    variance/=(double)(nbIndividus-1);
    
    if(variance>varianceLimite) {
      for(j=0;j<nbIndividus;j++) {
        individus[j]->valeurs[*nbDimensionsUtilisees]=individus[j]->valeurs[i];
      }
      (*nbDimensionsUtilisees)++;
    }
  }
}

/******************************************************/
/*                                                    */
/*Procedure de filtrage des dimensions ou celles ayant*/ 
/*des variances sont trop petites sont eliminees      */
/*                                                    */
/******************************************************/

void filtreDimensionsDeuxEnsembles(int nbIndividus,int nbDimensionsOrigine,int *nbDimensionsUtilisees,
				   individu_t **individus,int nbIndividus2,individu_t **individus2) {
  int i,j;
  double temp,moyenne,variance,varianceLimite=0.00000001;

  *nbDimensionsUtilisees=0;
  for(i=0;i<nbDimensionsOrigine;i++) {
    moyenne=0;
    for(j=0;j<nbIndividus;j++) {
      moyenne+=individus[j]->valeurs[i];
    }
    moyenne/=(double)nbIndividus;

    variance=0;
    for(j=0;j<nbIndividus;j++) {
      temp=individus[j]->valeurs[i]-moyenne;
      variance+=temp*temp;
    }
    variance/=(double)(nbIndividus-1);
    
    if(variance>varianceLimite) {
      for(j=0;j<nbIndividus;j++) {
        individus[j]->valeurs[*nbDimensionsUtilisees]=individus[j]->valeurs[i]/sqrt(variance);
      }
      for(j=0;j<nbIndividus2;j++) {
        individus2[j]->valeurs[*nbDimensionsUtilisees]=individus2[j]->valeurs[i]/sqrt(variance);
      }
      (*nbDimensionsUtilisees)++;
    }
  }
}

/*********************************************/
/*                                           */
/*Procedure de filtrage des individus suivant*/ 
/*une valeur seuil et le type des donnees    */
/*                                           */
/*********************************************/

void filtreIndividus(int *nbIndividus,int nbDimensions,individu_t **individus,
		      int typeDonnees,double distanceMin,int nbClustersSelection)
{
  /*declaration de variables*/
  int i,j,k,*individusElimines,nbIndividusElimines,compteur;
  double distance;
  /*fin declaration de variables*/

  /*allocation memoire*/
  individusElimines=(int *)malloc(sizeof(int)*(*nbIndividus));
  /*fin allocation memoire*/

  nbIndividusElimines=0;
 
  for(i=0;i<*nbIndividus;i++)
    {
      individusElimines[i]=NON;
      individus[i]->nbIndividusSimilaires=0;
      individus[i]->individusSimilaires=NULL;
    }
  for(i=0;i<*nbIndividus;i++)
    {
      for(j=i+1;j<*nbIndividus;j++)
	{
	  /*COORDONNEES*/
	  if(typeDonnees==COORDONNEES)
	    {
	      distance=0;
	      for(k=0;k<nbDimensions;k++)
		{
		  distance+=(individus[i]->valeurs[k]-
			     individus[j]->valeurs[k])*
		    (individus[i]->valeurs[k]-
		     individus[j]->valeurs[k]);
		}
	      distance=sqrt(distance);
	    }
	  /*ALIGNEMENT*/
	  else if(typeDonnees==ALIGNEMENT)
	    {
	     
		  distance=1.0-individus[i]->valeurs[j];
		
	    }
	
	  if((distance<distanceMin)&&(individusElimines[i]==NON)&&
	     (individusElimines[j]==NON))
	    {
	      nbIndividusElimines++;
	      individusElimines[j]=OUI;
	      individus[i]->nbIndividusSimilaires++;
	      individus[i]->individusSimilaires=
		(individu_t **)realloc(individus[i]->individusSimilaires,sizeof(individu_t *)
				       *individus[i]->nbIndividusSimilaires);
	      
	      individus[i]->individusSimilaires[individus[i]->nbIndividusSimilaires-1]=
		individus[j];
	    }
	}
    }

  compteur=0;
  for(i=0;i<*nbIndividus;i++)
    {
      if(individusElimines[i]==NON)
	{
	  individus[compteur]=individus[i];
	  compteur++;
	}
    }
  *nbIndividus-=nbIndividusElimines;

  /*desallocation memoire*/
  free(individusElimines);
  /*fin desallocation memoire*/
}

