#include "main.h"
#include "lecfi.h"
#include "kmeans.h"
#include "ecrfi.h"
#include "divide.h"
#include "alignment.h"
#include "treeward.h"
#include "treecut.h"
#include "treedpc.h"
#include "pretraitements.h"
#include "mds.h"

/**********************************************/
/*                                            */
/*Programme principal du package de clustering*/
/*                                            */
/**********************************************/

int main(int argc,char **argv)
{
  /*declaration de variables*/
  int i,j,typeDonnees,clusteringMethod,nbClustersSelection,nbClustersSelected=0;
  int typeDensite,weighting,donneesStandardisees,standardisation;
  int writeCoordinates,filtrageIndividus,longueurAlignement,nbIndividus;
  int nbDimensionsOrigine,nbDimensionsDemandees,nbDimensionsUtilisees;
  int nbClusters,nbIterationsMax,nbVoisins,individusTousIdentiques,indiceIndividu;
  int bandwidthType,accelerationCalculs,kPlusProchesVoisins;
  int nbDimensionsMax=20,diag,choixDistribution,methodeEstimationParametres,nbClustersMax=-1;
  char *buf,*fichierEntree,**sequences,nomFichier[TAILLE_NOM];
  char *outputFileClustering,*outputFile_tfa;
  char *arbreTexte;
  double distanceMin,seuilDissimilarite,**distances,bandwidth,scale,defaultBandwidth=1.0,proba=0.01;
  individu_t **individus;
  noeud_t *arbre;
  /*fin declaration de variables*/

//	printf("df : %f\n",bessel_j(10,5));

  /************************************/
  /*Traitement des parametres d'entree*/ 
  /************************************/

  /*initialisation de la generation aleatoire de nombres*/
  set_seed(getpid(),(int)floor(10000.0*(double)rand()/(double)RAND_MAX));

  /*parametres par defaut*/
  weighting=NON;
  donneesStandardisees=NON;
  standardisation=NON;
  writeCoordinates=NON;
  filtrageIndividus=NON;
  outputFileClustering=NULL;
  outputFile_tfa=NULL;
  typeDensite=DENSITE1;
  nbIterationsMax=-1;
  clusteringMethod=-1;
  nbVoisins=-1;
  nbDimensionsDemandees=-1;
  /*fin parametres par defaut*/

  if(argc==1) {
     helpMessageDPC();
  }

  fichierEntree=strdup(argv[1]);

  if((strcmp(fichierEntree,"-help")==0)||(strcmp(fichierEntree,"-h")==0)) {
      helpMessageDPC();
    }
  if(fopen(fichierEntree,"r")==NULL) {
  	printf("File %s does not exist !\n",fichierEntree);
  	exit(0);
  }
  for(i=2;i<argc;i++)
    {
      if(strstr(argv[i],"-dt=")!=NULL)
	{
	  buf=strstr(argv[i],"=")+1;
	  if(strcmp(buf,"coordinates")==0)
	    {
	      typeDonnees=COORDONNEES;
	    }
	  else if(strcmp(buf,"alignment")==0)
	    {
	      typeDonnees=ALIGNEMENT;
	    }
	  else 
	    {
	      printf("bad argument for -data_types !\n");
	      exit(1);
	    }
	}
      else if(strstr(argv[i],"-cm=")!=NULL) {
	  buf=strstr(argv[i],"=")+1;
	  if(strcmp(buf,"kmeans")==0) {
	      clusteringMethod=KMEANS;
	    }
	  else if(strcmp(buf,"hierar")==0) {
	      clusteringMethod=WARD;
	    }
	  else {
	      printf("bad argument for -clusteringMethod !\n");
	      exit(1);
	    }
	}

	else if(strstr(argv[i],"-proba=")!=NULL)
	{
	  buf=strstr(argv[i],"=")+1;
	  proba=atof(buf);
	}
	else if(strstr(argv[i],"-nbcmax=")!=NULL)
	{
	  buf=strstr(argv[i],"=")+1;
	  nbClustersMax=atoi(buf);
	}
      else if(strstr(argv[i],"-dt1")!=NULL)
	{
	  typeDensite=DENSITE1;
	}
      else if(strstr(argv[i],"-dt2")!=NULL)
	{
	  typeDensite=DENSITE2;
	}
      else if(strstr(argv[i],"-weighting")!=NULL)
	{
	  weighting=OUI;
	}
      else if(strstr(argv[i],"-standardized_data")!=NULL)
	{
	  donneesStandardisees=OUI;
	}
      else if(strstr(argv[i],"-standardization")!=NULL)
	{
	  standardisation=OUI;
	  donneesStandardisees=OUI;
	}
      else if(strstr(argv[i],"-wc")!=NULL)
	{
	  writeCoordinates=OUI;
	}
       else if(strstr(argv[i],"-nbsim=")!=NULL)
	{
	  nbIterationsMax=atof(strstr(argv[i],"=")+1);
	}
       else if(strstr(argv[i],"-otfa=")!=NULL)
	{
	  outputFile_tfa=strdup(strstr(argv[i],"=")+1);
	} 
       else if(strstr(argv[i],"-oclu=")!=NULL)
	{
	  outputFileClustering=strdup(strstr(argv[i],"=")+1);
	} 
       else if(strstr(argv[i],"-neighb=")!=NULL)
	{
	  nbVoisins=atoi(strstr(argv[i],"=")+1);
	}
       else if(strstr(argv[i],"-dim=")!=NULL)
	{
	  nbDimensionsDemandees=atoi(strstr(argv[i],"=")+1);
	}
      else if(strstr(argv[i],"-fd=")!=NULL)
	{
	  filtrageIndividus=OUI;
	  distanceMin=atof(strstr(argv[i],"=")+1);
	} 
    }
  

  /****************************************/
  /*Fin traitement des parametres d'entree*/
  /****************************************/

  /*********************/
  /*                   */
  /*LECTURE DES DONNEES*/
  /*                   */
  /*********************/

  /*lecture des donnees*/
  if(typeDonnees==ALIGNEMENT)
    {
      /*lecture du fichier de l'alignement multiple*/
      lectureFichierAlignement(fichierEntree,&longueurAlignement,&nbIndividus,&individus,&sequences);
     
    
	  if(nbDimensionsDemandees==-1) {
	    nbDimensionsUtilisees=2;
	  } else {
	    nbDimensionsUtilisees=nbDimensionsDemandees;
	  }
	  if(nbVoisins==-1) {
	    nbVoisins=nbDimensionsUtilisees+1;
	  }
	 
	  /*calcul des pourcentages d'identite entre les sequences*/

	  /*allocation memoire*/
	  distances=(double **)malloc(sizeof(double *)*nbIndividus);
	  for(i=0;i<nbIndividus;i++) {
	    distances[i]=(double *)malloc(sizeof(double)*nbIndividus);
	  }
	  /*fin allocation memoire*/
	  calculDistancesApartSequences(nbIndividus,longueurAlignement,
					sequences,distances);

	  distancesToCoordonnees(nbIndividus,individus,distances,nbDimensionsUtilisees);

	  /*  for(i=0;i<nbIndividus;i++) {
	    for(j=0;j<nbDimensionsUtilisees;j++) {
	      printf("%.3f ",individus[i]->valeursTraitees[j]);
	    }
	    printf("\n");
	  }
	  getchar();*/
	  /*desallocation memoire*/
	  for(i=0;i<nbIndividus;i++) {
	    free(distances[i]);
	  }
	  free(distances);
	  /*fin desallocation memoire*/
	
    }
   else  {
      /*lecture du fichier de coordonnees*/
      lectureFichierCoordonnees(&nbIndividus,&nbDimensionsOrigine,&individus,fichierEntree,typeDonnees);
      filtreDimensions(nbIndividus,nbDimensionsOrigine,&nbDimensionsUtilisees,individus);
     }
  
  /***************************/
  /*                         */
  /*PRETRAITEMENT DES DONNEES*/
  /*                         */
  /***************************/

  if(filtrageIndividus==OUI) {
    /*filtrage des individus suivant la valeur seuil donnee et le type des donnees*/
    filtreIndividus(&nbIndividus,nbDimensionsUtilisees,individus,typeDonnees,distanceMin,nbClustersSelection);
  }
  if((typeDonnees==COORDONNEES)&&(nbDimensionsDemandees>0)) {
    /*reduction de dimension*/
    reductionDimension(nbIndividus,individus,nbDimensionsOrigine,nbDimensionsDemandees);
    nbDimensionsUtilisees=nbDimensionsDemandees;
  } 
  
  if(standardisation==OUI)
    {
      /*on normalise les individus*/   
      standardisationIndividus(nbIndividus,nbDimensionsUtilisees,individus);
    }

  /************/
  /*          */
  /*CLUSTERING*/
  /*          */
  /************/
 
  /*clustering method : KMEANS */
  if(clusteringMethod==KMEANS) {
    /*the number of clusters is fixed*/
    divide(nbIndividus,nbDimensionsUtilisees,&nbClusters,individus,typeDonnees,
	       clusteringMethod,donneesStandardisees,typeDensite,nbVoisins);
  }  
  /*clustering method : WARD*/
  else if(clusteringMethod==WARD)
    {
      classificationWard(nbIndividus,nbDimensionsUtilisees,individus,typeDonnees,&arbre);
      
 
	  decoupageArbreDPC(nbIndividus,nbDimensionsUtilisees,individus,arbre,&nbClusters,
			      typeDonnees,nbClustersSelection,donneesStandardisees,
			      typeDensite,nbVoisins);
	
    }
 
 
  /************************/
  /*                      */
  /*ECRITURE DES RESULTATS*/
  /*                      */
  /************************/

  /*ecriture du fichier resultat presentant les clusters*/
  if(outputFileClustering==NULL)
    {
      strcpy(nomFichier,fichierEntree);
      if(strstr(nomFichier,".")!=NULL)
	{
	  sprintf(strstr(nomFichier,"."),".clu");
	}
      else
	{
	  strcat(nomFichier,".clu");
	}
    }
  else
    {
      strcpy(nomFichier,outputFileClustering);
    }

  /*ecriture du fichier resultat*/

//  printf("avant ecriture fichier clusters\n");
  if(nbDimensionsDemandees>0) {
    ecritureFichierClusters(nbIndividus,nbDimensionsOrigine,individus,nbClusters,
			    nbClustersSelection,writeCoordinates,nomFichier);
  } else {
    ecritureFichierClusters(nbIndividus,nbDimensionsUtilisees,individus,nbClusters,
			      nbClustersSelection,writeCoordinates,nomFichier);
  }

  if(typeDonnees==ALIGNEMENT)
    {
      if(outputFile_tfa==NULL)
	{
	  strcpy(nomFichier,fichierEntree);
	  if(strstr(nomFichier,".")!=NULL)
	  {  
	    sprintf(strstr(nomFichier,"."),"2.tfa");
	   }
          else
	  {
	    strcat(nomFichier,"2.tfa");
	  }
	}
      else
	{
	  strcpy(nomFichier,outputFile_tfa);
	}
     
    
      /*ecriture du fichier de l'alignement multiple avec les sequences*/
      /*reordonnees suivant leurs groupes*/
      ecritureFichierClustersAlignement(nomFichier,longueurAlignement,nbIndividus,
					   individus,sequences,nbClusters);
    }

  /*desallocation memoire*/
  for(i=0;i<nbIndividus;i++)
    {
      free(individus[i]->nom);
      free(individus[i]->valeurs);
      free(individus[i]);
    }
  free(individus);
  /*fin desallocation memoire*/
 // printf("main exit\n");
    exit(0);
}

void helpMessageDPC() {
      printf("Program usage : dpc file options\n\n");
      printf("********************OPTIONS BELOW********************\n\n");
      printf("-dt=[coordinates|alignment|distances] (dt stands for data_type)\n");
      printf("-cm=[kmeans|hierar] (cm stands for clusteringMethod, mm for mixture models))\n");
      printf("-dim=[dimension] (for dimension reduction when dealing with alignments and using DPC equanl to 2 by default)\n");
      printf("[-dt1|-dt2][-standardization][-standardized_data][-wc] (dt1 stands for density1 and wc for writeCoordinates)\n");
      printf("[-nbsim=nbsimulations]\n");
      printf("[-otfa=outputFile for alignment]\n");
      printf("[-oclu=outputFile for clustering]\n");
      exit(0);
}


