#include "main.h"
#include "symeig.h"
#include "matrice.h"
#include "dissimilarities_clustering.h"

/********************************************/
/*                                          */
/*Procedure de standardisation des individus*/
/*                                          */
/********************************************/

void standardisationIndividus(int nbIndividus,int nbDimensions,individu_t **individus)
{
  /*declaration de variables*/
  int i,j;
  double moyenne,ecartType;
  /*fin declaration de variables*/
  
  for(i=0;i<nbIndividus;i++)
    {
      moyenne=0;
      for(j=0;j<nbDimensions;j++)
	{
	  moyenne+=individus[i]->valeurs[j];
	}
      moyenne/=(double)nbDimensions;

      ecartType=0;
      for(j=0;j<nbDimensions;j++)
	{
	  ecartType+=(individus[i]->valeurs[j]-moyenne)*(individus[i]->valeurs[j]-moyenne);
	}
      ecartType=sqrt(ecartType/(double)nbDimensions);

      if(ecartType<DBL_MIN)
	{
	  printf("Standard deviation of %s is too low : %f\n",individus[i]->nom,ecartType);
	  exit(1);
	}

      for(j=0;j<nbDimensions;j++)
	{
	  individus[i]->valeurs[j]=(individus[i]->valeurs[j]-moyenne)/ecartType;
	}
    }
}

/*************************************/
/*                                   */
/*Procedure de reduction de dimension*/
/*                                   */
/*************************************/

void reductionDimension(int nbIndividus,individu_t **individus,int nbDimensions,
			int nbDimensionsDemandees) {
  /*declaration de variables*/
  int i,j,k;
  double temp,*moyenne,**matriceCovariance,**vecteursPropres,*valeursPropres;;
  double **nouvellesCoordonnees;
  /*fin declaration de variables*/
  
  /*allocation memoire*/
  moyenne=(double *)malloc(sizeof(double)*nbDimensions);
  matriceCovariance=(double **)malloc(sizeof(double *)*nbDimensions);
  for(i=0;i<nbDimensions;i++) {
    matriceCovariance[i]=(double *)malloc(sizeof(double)*nbDimensions);
  }
  valeursPropres=(double *)malloc(sizeof(double)*nbDimensions);
  vecteursPropres=(double **)malloc(sizeof(double *)*nbDimensions);
  for(i=0;i<nbDimensions;i++) {
    vecteursPropres[i]=(double *)malloc(sizeof(double)*nbDimensions);
  }
  nouvellesCoordonnees=(double **)malloc(sizeof(double *)*nbIndividus);
  for(i=0;i<nbIndividus;i++) {
    nouvellesCoordonnees[i]=(double *)malloc(sizeof(double)*nbDimensionsDemandees);
  }
  /*fin allocation memoire*/
  
  /*calcul de la moyenne*/
  for(j=0;j<nbDimensions;j++)
    {
      moyenne[j]=0;
    }
  for(i=0;i<nbIndividus;i++)
    {
      for(j=0;j<nbDimensions;j++)
	{
	  moyenne[j]+=individus[i]->valeurs[j];
	}
    }
  for(j=0;j<nbDimensions;j++)
    {
      moyenne[j]/=(double)nbIndividus;
    }
  
  /*calcul de la matrice de variance-covariance*/ 
  for(i=0;i<nbDimensions;i++)
    {
      for(j=0;j<nbDimensions;j++)
	{
	  matriceCovariance[i][j]=0;
	}
    }
  for(k=0;k<nbIndividus;k++)
    {
      for(i=0;i<nbDimensions;i++)
	{ 
	  temp=individus[k]->valeurs[i]-moyenne[i];
	  for(j=i;j<nbDimensions;j++)
	    {
	      matriceCovariance[i][j]+=temp*(individus[k]->valeurs[j]-moyenne[j]);
	    }
	}	 
    }
   for(i=0;i<nbDimensions;i++)
    {
      for(j=i;j<nbDimensions;j++)
	{
	  matriceCovariance[i][j]/=(double)(nbIndividus-1);
	  
	  matriceCovariance[j][i]=matriceCovariance[i][j];
	}
    }	

   /*Principal Component Analysis*/
   /*on calcule les vecteurs propres de la matrice de covariance*/
   symeig(matriceCovariance,nbDimensions,vecteursPropres,valeursPropres);

   for(i=0;i<nbDimensions;i++) {
     printf("eigenvalues : %f\n",valeursPropres[i]);
   }

   /*calcul des nouvelles coordonnees*/
   for(i=0;i<nbIndividus;i++) {
     for(j=0;j<nbDimensionsDemandees;j++) {
       nouvellesCoordonnees[i][j]=0;
       for(k=0;k<nbDimensions;k++) {
	 nouvellesCoordonnees[i][j]+=vecteursPropres[k][nbDimensions-1-j]*
	   (individus[i]->valeurs[k]-moyenne[k]);
       }
       nouvellesCoordonnees[i][j]+=moyenne[j];
     }
   }

   /*mise a jour des coordonnees des individus*/
   for(i=0;i<nbIndividus;i++) {
     for(j=0;j<nbDimensionsDemandees;j++) {
       individus[i]->valeurs[j]=nouvellesCoordonnees[i][j];
     }
   }

   /*desallocation memoire*/
   free(moyenne);
   free(valeursPropres);
   for(i=0;i<nbDimensions;i++) {
     free(matriceCovariance[i]);
     free(vecteursPropres[i]);
   }
   free(matriceCovariance);
   free(vecteursPropres);
   for(i=0;i<nbIndividus;i++) {
     free(nouvellesCoordonnees[i]);
   }
   free(nouvellesCoordonnees);
   /*fin desallocation memoire*/
}

